@@ -43,9 +43,18 @@ typedef struct {
 } Clp_ArgType;
 
 
+typedef struct {
+
+  int pos;
+  int neg;
+
+} Clp_LongMinMatch;
+
+
 struct Clp_Internal {
-  
+
   Clp_Option *opt;
+  Clp_LongMinMatch *long_min_match;
   int nopt;
   
   Clp_ArgType *argtype;
@@ -91,12 +100,14 @@ struct Clp_ParserState {
 };
 
 
-typedef struct {
+typedef struct Clp_StringList {
+
+  Clp_Option *items;
+  Clp_LongMinMatch *long_min_match;
+  int nitems;
   
   int allow_int;
-  int nitems;
   int nitems_invalid_report;
-  Clp_Option *items;
   
 } Clp_StringList;
 
@@ -149,9 +160,11 @@ Clp_NewParser(int argc, char * const argv[], int nopt, Clp_Option *opt)
   int i;
   Clp_Parser *clp = (Clp_Parser *)malloc(sizeof(Clp_Parser));
   Clp_Internal *cli = (Clp_Internal *)malloc(sizeof(Clp_Internal));
-  if (!clp || !cli) goto failed;
+  Clp_LongMinMatch *lmm = (Clp_LongMinMatch *)malloc(sizeof(Clp_LongMinMatch) * nopt);
+  if (!clp || !cli || !lmm) goto failed;
   
   clp->internal = cli;
+  cli->long_min_match = lmm;
   
   /* Get rid of negative option_ids, which are internal to CLP */
   for (i = 0; i < nopt; i++)
@@ -161,10 +174,6 @@ Clp_NewParser(int argc, char * const argv[], int nopt, Clp_Option *opt)
       nopt--;
       i--;
     }
-
-  /* Check for duplicated short options */
-  check_duplicated_short_options(nopt, opt, 0);
-  check_duplicated_short_options(nopt, opt, 1);
   
   /* Massage the options to make them usable */
   for (i = 0; i < nopt; i++) {
@@ -182,18 +191,22 @@ Clp_NewParser(int argc, char * const argv[], int nopt, Clp_Option *opt)
     /* Options that start with `no-' should be changed to OnlyNegated */
     if (opt[i].long_name && strncmp(opt[i].long_name, "no-", 3) == 0) {
       opt[i].long_name += 3;
-      opt[i].flags |= Clp_OnlyNegated | Clp_Negate;
+      opt[i].flags |= Clp_Negate | Clp_OnlyNegated;
     }
   }
   
+  /* Check for duplicated short options */
+  check_duplicated_short_options(nopt, opt, 0);
+  check_duplicated_short_options(nopt, opt, 1);
+  
   /* Calculate long options' minimum unambiguous length */
   for (i = 0; i < nopt; i++)
     if (opt[i].long_name && !TEST(&opt[i], Clp_OnlyNegated))
-      opt[i].long_min_match = calculate_long_min_match
+      lmm[i].pos = calculate_long_min_match
 	(nopt, opt, i, Clp_OnlyNegated, 0);
   for (i = 0; i < nopt; i++)
     if (opt[i].long_name && TEST(&opt[i], Clp_Negate))
-      opt[i].negated_long_min_match = calculate_long_min_match
+      lmm[i].neg = calculate_long_min_match
 	(nopt, opt, i, Clp_Negate, Clp_Negate);
   
   /* Set up clp->internal */
@@ -236,6 +249,7 @@ Clp_NewParser(int argc, char * const argv[], int nopt, Clp_Option *opt)
   if (cli && cli->argtype) free(cli->argtype);
   if (cli) free(cli);
   if (clp) free(clp);
+  if (lmm) free(lmm);
   return 0;
 }
 
@@ -255,10 +269,12 @@ Clp_DeleteParser(Clp_Parser *clp)
     if (cli->argtype[i].func == parse_string_list) {
       Clp_StringList *clsl = (Clp_StringList *)cli->argtype[i].thunk;
       free(clsl->items);
+      free(clsl->long_min_match);
       free(clsl);
     }
   
   free(cli->argtype);
+  free(cli->long_min_match);
   free(cli);
   free(clp);
 }
@@ -338,9 +354,12 @@ Clp_SetOptionChar(Clp_Parser *clp, int c, int option_type)
       for (i = 0; i < cli->nopt; i++)
 	have_short[cli->opt[i].short_name] = 1;
       for (i = 0; i < cli->nopt; i++)
-	if (cli->opt[i].long_name && cli->opt[i].long_min_match == 1
-	    && have_short[ (unsigned char)cli->opt[i].long_name[0] ])
-	  cli->opt[i].long_min_match++;
+	if (cli->opt[i].long_name && cli->long_min_match[i].pos == 1) {
+	  /* if `--Cxxxx's short name is `-C', keep long_min_match == 1 */
+	  unsigned char first = (unsigned char)cli->opt[i].long_name[0];
+	  if (have_short[first] && first != cli->opt[i].short_name)
+	    cli->long_min_match[i].pos++;
+	}
       cli->both_short_and_long = 1;
     }
   }
@@ -414,6 +433,7 @@ argcmp(const char *ref, const char *arg, int min_match)
 
 static int
 find_prefix_opt(const char *arg, int nopt, Clp_Option *opt,
+		Clp_LongMinMatch *lmmvec,
 		int *ambiguous, int *ambiguous_values, int negated)
      /* Looks for an unambiguous match of `arg' against one of the long
         options in `opt'. Returns positive if it finds one; otherwise, returns
@@ -428,7 +448,7 @@ find_prefix_opt(const char *arg, int nopt, Clp_Option *opt,
 	|| (!negated && TEST(&opt[i], Clp_OnlyNegated)))
       continue;
     
-    lmm = (negated ? opt[i].negated_long_min_match : opt[i].long_min_match);
+    lmm = (negated ? lmmvec[i].neg : lmmvec[i].pos);
     len = argcmp(opt[i].long_name, arg, lmm);
     if (len > 0)
       return i;
@@ -459,6 +479,7 @@ Clp_AddType(Clp_Parser *clp, int type_id, int flags,
 {
   int i;
   Clp_Internal *cli = clp->internal;
+  Clp_ArgType *new_argtype;
   int nargtype = cli->nargtype;
   assert(nargtype);
   
@@ -466,9 +487,10 @@ Clp_AddType(Clp_Parser *clp, int type_id, int flags,
   
   while (nargtype <= type_id)
     nargtype *= 2;
-  cli->argtype = (Clp_ArgType *)
+  new_argtype = (Clp_ArgType *)
     realloc(cli->argtype, sizeof(Clp_ArgType) * nargtype);
-  if (!cli->argtype) return 0;
+  if (!new_argtype) return 0;
+  cli->argtype = new_argtype;
   
   for (i = cli->nargtype; i < nargtype; i++)
     cli->argtype[i].func = 0;
@@ -497,7 +519,7 @@ parse_int(Clp_Parser *clp, const char *arg, int complain, void *thunk)
 {
   char *val;
   int base = 10;
-  if (arg[0] == '0' && arg[1] == 'x') {
+  if (arg[0] == '0' && (arg[1] == 'x' || arg[1] == 'X')) {
     base = 16;
     arg += 2;
   }
@@ -580,21 +602,20 @@ parse_string_list(Clp_Parser *clp, const char *arg, int complain, void *thunk)
   
   /* actually look for a string value */
   index = find_prefix_opt
-    (arg, sl->nitems, sl->items, &ambiguous, ambiguous_values, 0);
+    (arg, sl->nitems, sl->items, sl->long_min_match,
+     &ambiguous, ambiguous_values, 0);
   if (index >= 0) {
     clp->val.i = sl->items[index].option_id;
     return 1;
   }
   
   if (sl->allow_int) {
-    char *x;
-    clp->val.i = strtol(arg, &x, 10);
-    if (*arg != 0 && *x == 0)
+    if (parse_int(clp, arg, 0, 0))
       return 1;
   }
   
   if (complain) {
-    char *complaint = (ambiguous ? "ambiguous" : "invalid");
+    char *complaint = (ambiguous ? "an ambiguous" : "not a valid");
     if (!ambiguous) {
       ambiguous = sl->nitems_invalid_report;
       for (index = 0; index < ambiguous; index++)
@@ -602,7 +623,7 @@ parse_string_list(Clp_Parser *clp, const char *arg, int complain, void *thunk)
     }
     return ambiguity_error
       (clp, ambiguous, ambiguous_values, sl->items, "",
-       "`%s' is an %s argument to `%O'", arg, complaint);
+       "`%s' is %s argument to `%O'", arg, complaint);
   } else
     return 0;
 }
@@ -614,14 +635,16 @@ finish_string_list(Clp_Parser *clp, int type_id, int flags,
 {
   int i;
   Clp_StringList *clsl = (Clp_StringList *)malloc(sizeof(Clp_StringList));
-  if (!clsl) return 0;
+  Clp_LongMinMatch *lmm = (Clp_LongMinMatch *)malloc(sizeof(Clp_LongMinMatch) * nitems);
+  if (!clsl || !lmm) goto error;
   
-  clsl->allow_int = (flags & Clp_AllowNumbers) != 0;
   clsl->items = items;
+  clsl->long_min_match = lmm;
   clsl->nitems = nitems;
+  clsl->allow_int = (flags & Clp_AllowNumbers) != 0;
   
   if (nitems < MAX_AMBIGUOUS_VALUES && nitems < itemscap && clsl->allow_int) {
-    items[nitems].long_name = "or any integer";
+    items[nitems].long_name = "any integer";
     clsl->nitems_invalid_report = nitems + 1;
   } else if (nitems > MAX_AMBIGUOUS_VALUES + 1)
     clsl->nitems_invalid_report = MAX_AMBIGUOUS_VALUES + 1;
@@ -629,14 +652,15 @@ finish_string_list(Clp_Parser *clp, int type_id, int flags,
     clsl->nitems_invalid_report = nitems;
   
   for (i = 0; i < nitems; i++)
-    items[i].long_min_match = calculate_long_min_match(nitems, items, i, 0, 0);
+    lmm[i].pos = calculate_long_min_match(nitems, items, i, 0, 0);
   
   if (Clp_AddType(clp, type_id, 0, parse_string_list, clsl))
     return 1;
-  else {
-    free(clsl);
-    return 0;
-  }
+
+ error:
+  if (clsl) free(clsl);
+  if (lmm) free(lmm);
+  return 0;
 }
 
 int
@@ -673,9 +697,11 @@ Clp_AddStringListType(Clp_Parser *clp, int type_id, int flags, ...)
     value = va_arg(val, int);
     
     if (nitems >= itemscap) {
+      Clp_Option *new_items;
       itemscap *= 2;
-      items = (Clp_Option *)realloc(items, sizeof(Clp_Option) * itemscap);
-      if (!items) goto error;
+      new_items = (Clp_Option *)realloc(items, sizeof(Clp_Option) * itemscap);
+      if (!new_items) goto error;
+      items = new_items;
     }
     
     items[nitems].long_name = name;
@@ -945,8 +971,8 @@ static Clp_Option *
 find_long(Clp_Parser *clp, char *arg)
      /* If arg corresponds to one of clp's options, finds that option &
 	returns it. If any argument is given after an = sign in arg, sets
-	clp->have_arg = 1 and clp->arg to that argument. Sets cli->ambiguous = 1
-	iff there was no match because the argument given was ambiguous. */
+	clp->have_arg = 1 and clp->arg to that argument. Sets cli->ambiguous
+	to 1 iff there was no match because the argument was ambiguous. */
 {
   Clp_Internal *cli = clp->internal;
   int value, len;
@@ -954,8 +980,9 @@ find_long(Clp_Parser *clp, char *arg)
   int first_negative_ambiguous;
   
   /* Look for a normal option. */
-  value = find_prefix_opt(arg, cli->nopt, opt, &cli->ambiguous,
-			  cli->ambiguous_values, clp->negated);
+  value = find_prefix_opt
+    (arg, cli->nopt, opt, cli->long_min_match,
+     &cli->ambiguous, cli->ambiguous_values, clp->negated);
   if (value >= 0)
     goto worked;
   
@@ -966,8 +993,9 @@ find_long(Clp_Parser *clp, char *arg)
   while (arg[0] == 'n' && arg[1] == 'o' && arg[2] == '-') {
     arg += 3;
     clp->negated = !clp->negated;
-    value = find_prefix_opt(arg, cli->nopt, opt, &cli->ambiguous,
-			    cli->ambiguous_values, clp->negated);
+    value = find_prefix_opt
+      (arg, cli->nopt, opt, cli->long_min_match,
+       &cli->ambiguous, cli->ambiguous_values, clp->negated);
     if (value >= 0)
       goto worked;
   }
@@ -983,8 +1011,7 @@ find_long(Clp_Parser *clp, char *arg)
   }
   
  worked:
-  cli->ambiguous = 0;
-  len = argcmp(opt[value].long_name, arg, opt[value].long_min_match);
+  len = argcmp(opt[value].long_name, arg, cli->long_min_match[value].pos);
   if (arg[len] == '=') {
     clp->have_arg = 1;
     clp->arg = arg + len + 1;
@@ -1007,7 +1034,6 @@ find_short(Clp_Parser *clp, int short_name)
 	    : !TEST(&opt[i], Clp_OnlyNegated)))
       return &opt[i];
   
-  cli->ambiguous = 0;
   return 0;
 }
 
@@ -1041,6 +1067,7 @@ Clp_Next(Clp_Parser *clp)
   
   /** Set up clp **/
   cli->current_option = 0;
+  cli->ambiguous = 0;
   
   /** Get the next argument or option **/
   if (!next_argument(clp, cli->option_processing ? 0 : 2))
@@ -1169,48 +1196,65 @@ typedef struct Clp_BuildString {
   char *text;
   char *pos;
   int capacity;
+  int bad;
 } Clp_BuildString;
 
 static Clp_BuildString *
 new_build_string(void)
 {
   Clp_BuildString *bs = (Clp_BuildString *)malloc(sizeof(Clp_BuildString));
+  if (!bs) goto bad;
   bs->text = (char *)malloc(256);
+  if (!bs->text) goto bad;
   bs->pos = bs->text;
   bs->capacity = 256;
+  bs->bad = 0;
   return bs;
+  
+ bad:
+  if (bs) free(bs);
+  return 0;
 }
 
 static void
 free_build_string(Clp_BuildString *bs)
 {
-  free(bs->text);
+  if (bs) free(bs->text);
   free(bs);
 }
 
-static void
+static int
 grow_build_string(Clp_BuildString *bs, int want)
 {
+  char *new_text;
   int ipos = bs->pos - bs->text;
-  while (want >= bs->capacity)
-    bs->capacity *= 2;
-  bs->text = (char *)realloc(bs->text, bs->capacity);
-  bs->pos = bs->text + ipos;
+  int new_capacity = bs->capacity;
+  while (want >= new_capacity)
+    new_capacity *= 2;
+  new_text = (char *)realloc(bs->text, new_capacity);
+  if (!new_text) {
+    bs->bad = 1;
+    return 0;
+  } else {
+    bs->text = new_text;
+    bs->pos = bs->text + ipos;
+    bs->capacity = new_capacity;
+    return 1;
+  }
 }
 
 #define ENSURE_BUILD_STRING(bs, space) \
-  do {									\
-    if (((bs)->pos - (bs)->text) + (space) >= (bs)->capacity)		\
-      grow_build_string((bs), ((bs)->pos - (bs)->text) + (space));	\
-  } while (0)
+  ((((bs)->pos - (bs)->text) + (space) >= (bs)->capacity)		\
+   || grow_build_string((bs), ((bs)->pos - (bs)->text) + (space)))
 
 static void
 append_build_string(Clp_BuildString *bs, const char *s, int l)
 {
   if (l < 0) l = strlen(s);
-  ENSURE_BUILD_STRING(bs, l);
-  memcpy(bs->pos, s, l);
-  bs->pos += l;
+  if (ENSURE_BUILD_STRING(bs, l)) {
+    memcpy(bs->pos, s, l);
+    bs->pos += l;
+  }
 }
 
 
@@ -1231,6 +1275,7 @@ Clp_VaOptionError(Clp_Parser *clp, Clp_BuildString *bs,
   const char *percent;
   
   if (!bs) bs = new_build_string();
+  if (!bs) return 0;
   append_build_string(bs, cli->program_name, -1);
   append_build_string(bs, ": ", 2);
   
@@ -1247,24 +1292,26 @@ Clp_VaOptionError(Clp_Parser *clp, Clp_BuildString *bs,
      
      case 'c': {
        int c = va_arg(val, int);
-       ENSURE_BUILD_STRING(bs, 4);
-       if (c >= 32 && c <= 126)
-	 *bs->pos++ = c;
-       else if (c < 32) {
-	 *bs->pos++ = '^';
-	 *bs->pos++ = c + 64;
-       } else {
-	 sprintf(bs->pos, "\\%03o", c);
-	 bs->pos += 4;
+       if (ENSURE_BUILD_STRING(bs, 4)) {
+	 if (c >= 32 && c <= 126)
+	   *bs->pos++ = c;
+	 else if (c < 32) {
+	   *bs->pos++ = '^';
+	   *bs->pos++ = c + 64;
+	 } else {
+	   sprintf(bs->pos, "\\%03o", c);
+	   bs->pos += 4;
+	 }
        }
        break;
      }
      
      case 'd': {
        int d = va_arg(val, int);
-       ENSURE_BUILD_STRING(bs, 32);
-       sprintf(bs->pos, "%d", d);
-       bs->pos = strchr(bs->pos, 0);
+       if (ENSURE_BUILD_STRING(bs, 32)) {
+	 sprintf(bs->pos, "%d", d);
+	 bs->pos = strchr(bs->pos, 0);
+       }
        break;
      }
      
@@ -1274,8 +1321,8 @@ Clp_VaOptionError(Clp_Parser *clp, Clp_BuildString *bs,
 	 append_build_string(bs, "(no current option!)", -1);
        else if (cli->current_short) {
 	 append_build_string(bs, cli->option_chars, -1);
-	 ENSURE_BUILD_STRING(bs, 1);
-	 *bs->pos++ = opt->short_name;
+	 if (ENSURE_BUILD_STRING(bs, 1))
+	   *bs->pos++ = opt->short_name;
        } else if (cli->negated_by_no) {
 	 append_build_string(bs, cli->option_chars, -1);
 	 append_build_string(bs, "no-", 3);
@@ -1288,14 +1335,15 @@ Clp_VaOptionError(Clp_Parser *clp, Clp_BuildString *bs,
      }
      
      case '%':
-      ENSURE_BUILD_STRING(bs, 1);
-      *bs->pos++ = '%';
+      if (ENSURE_BUILD_STRING(bs, 1))
+	*bs->pos++ = '%';
       break;
       
      default:
-      ENSURE_BUILD_STRING(bs, 2);
-      *bs->pos++ = '%';
-      *bs->pos++ = *percent;
+      if (ENSURE_BUILD_STRING(bs, 2)) {
+	*bs->pos++ = '%';
+	*bs->pos++ = *percent;
+      }
       break;
       
     }
@@ -1308,6 +1356,22 @@ Clp_VaOptionError(Clp_Parser *clp, Clp_BuildString *bs,
   return bs;
 }
 
+static void
+do_error(Clp_Parser *clp, Clp_BuildString *bs)
+{
+  char *text;
+  if (bs && !bs->bad) {
+    *bs->pos = 0;
+    text = bs->text;
+  } else
+    text = "out of memory\n";
+  
+  if (clp->internal->error_handler != 0)
+    (*clp->internal->error_handler)(text);
+  else
+    fputs(text, stderr);
+}
+
 int
 Clp_OptionError(Clp_Parser *clp, const char *fmt, ...)
 {
@@ -1316,13 +1380,7 @@ Clp_OptionError(Clp_Parser *clp, const char *fmt, ...)
   va_start(val, fmt);
   bs = Clp_VaOptionError(clp, 0, fmt, val);
   va_end(val);
-  
-  *bs->pos = 0;
-  if (clp->internal->error_handler != 0)
-    (*clp->internal->error_handler)(bs->text);
-  else
-    fputs(bs->text, stderr);
-  
+  do_error(clp, bs);
   free_build_string(bs);
   return 0;
 }
@@ -1337,6 +1395,7 @@ ambiguity_error(Clp_Parser *clp, int ambiguous, int *ambiguous_values,
   va_list val;
   va_start(val, fmt);
   bs = Clp_VaOptionError(clp, 0, fmt, val);
+  if (!bs) goto done;
   
   append_build_string(bs, clp->internal->program_name, -1);
   append_build_string(bs, ": (Possibilities are", -1);
@@ -1361,12 +1420,8 @@ ambiguity_error(Clp_Parser *clp, int ambiguous, int *ambiguous_values,
   append_build_string(bs, ".)\n", -1);
   va_end(val);
   
-  *bs->pos = 0;
-  if (clp->internal->error_handler != 0)
-    (*clp->internal->error_handler)(bs->text);
-  else
-    fputs(bs->text, stderr);
-  
+ done:
+  do_error(clp, bs);
   free_build_string(bs);
   return 0;
 }
