@@ -27,7 +27,6 @@ unmark_colors(Gif_Colormap *gfcm)
       gfcm->col[i].haspixel = 0;
 }
 
-
 void
 unmark_colors_2(Gif_Colormap *gfcm)
 {
@@ -70,8 +69,11 @@ mark_used_colors(Gif_Image *gfi, Gif_Colormap *gfcm)
   /* Mark the transparent color specially */
   if (gfi->transparent >= 0 && gfi->transparent < ncol)
     col[gfi->transparent].haspixel = 2;
-  else
-    gfi->transparent = -1;
+  /* 1.Aug.1999 - DO NOT clear transparent index if gfi->transparent isn't
+     within bounds! Some smart GIF optimizers will write a small colormap, not
+     including the transparent color. */
+  /* 19.Aug.1999 - Don't need to set col[gfi->transparent].haspixel to 2 even
+     if it's not within bounds! */
 }
 
 
@@ -168,14 +170,14 @@ merge_colormap_if_possible(Gif_Colormap *dest, Gif_Colormap *src)
   dest->ncol = ndestcol;
   dest->userflags = dest_userflags;
   return 1;
-
+  
   /* failure: a local colormap is required */
  local_colormap_required:
   if (warn_local_colormaps == 1) {
     static int context = 0;
     warning("so many colors that local colormaps were required");
     if (!context)
-      warning("(You may want to try `--colors 256'.)");
+      warncontext("(You may want to try `--colors 256'.)");
     warn_local_colormaps = 2;
     context = 1;
   }
@@ -248,9 +250,9 @@ merge_image(Gif_Stream *dest, Gif_Stream *src, Gif_Image *srci)
   imagecm = islocal ? srci->local : src->global;
   if (!imagecm)
     fatal_error("no global or local colormap for source image");
-  imagecol = imagecm->col;
   
   mark_used_colors(srci, imagecm);
+  imagecol = imagecm->col;	/* may be changed by mark_used_colors */
   
   /* map[old_pixel_value] == new_pixel_value */
   for (i = 0; i < 256; i++)
@@ -274,7 +276,9 @@ merge_image(Gif_Stream *dest, Gif_Stream *src, Gif_Image *srci)
       if (imagecol[i].haspixel) {
 	map[i] = ncol;
 	if (ncol != i) trivial_map = 0;
-	used[ncol] = 1;
+	/* 19.Aug.1999- BUGFIX! color is only used if it was opaque */
+	if (imagecol[i].haspixel == 1)
+	  used[ncol] = 1;
 	localcm->col[ ncol++ ] = imagecol[i];
       }
     localcm->ncol = ncol;
@@ -292,12 +296,19 @@ merge_image(Gif_Stream *dest, Gif_Stream *src, Gif_Image *srci)
 	if (!used[i])
 	  found_transparent = i;
     
-    if (found_transparent < 0) {
+    /* 1.Aug.1999 - Allow for the case that the transparent index is bigger
+       than the number of colors we've created thus far. */
+    if (found_transparent < 0 || found_transparent >= destcm->ncol) {
       Gif_Color *c;
-      found_transparent = destcm->ncol++;
+      found_transparent = destcm->ncol;
+      /* 1.Aug.1999 - Don't update destcm->ncol -- we want the output colormap
+         to be as small as possible. */
       c = &destcm->col[found_transparent];
-      *c = imagecol[srci->transparent];
-      assert(c->haspixel == 2);
+      if (srci->transparent < imagecm->ncol)
+	*c = imagecol[srci->transparent];
+      else
+	c->haspixel = 2;
+      assert(c->haspixel == 2 && found_transparent < 256);
     }
     
     map[srci->transparent] = found_transparent;
